# plotLightcurve.py
#
# a code to plot a multi-frequency lightcurve for a source

# Import all the dependencies and generic setup
import os
import scipy as sp
import numpy as np
import pandas as pd
from sqlalchemy import *
from sqlalchemy.orm import relationship
import tkp.db
import logging
logging.basicConfig(level=logging.INFO)
query_loglevel = logging.WARNING  # Set to INFO to see queries, otherwise WARNING
import sys
sys.path.append('../')
from dblogin import * # This file contains all the variables required to connect to the database
from databaseTools import dbtools
from tools import tools
from plotting import plot_varib_params as pltvp
import matplotlib
import matplotlib.pyplot as plt
import pylab
pylab.rcParams['legend.loc'] = 'best'
from matplotlib.ticker import NullFormatter
from matplotlib.font_manager import FontProperties
from astropy import units as u
from astropy.coordinates import SkyCoord
from datetime import datetime

from tkp.db.model import Extractedsource
from tkp.db.model import Assocxtrsource
from tkp.db.model import Image

# The input database, dataset and thresholds
sourceID=3814
title = 'NVSS J182029+063419'
srcIDs = pd.DataFrame(data=[[3814,29,'0.963 GHz','#e41a1c','o'],
                                [4258,30,'1.177 GHz','#a65628','s'],
                                [4708,31,'1.391 GHz','#377eb8','v'],
                                [5084,32,'1.605 GHz','#984ea3','D']],
                          columns = ['srcID','datasetID','freq','colour','marker'])

database = 'AR_testing4'
outname =  str(sourceID)+'_lightcurve.png' # name of the plotdata datafile

freqs = pd.DataFrame(data=[[0.963,'#e41a1c','o',0.0838],
                                [1.177,'#a65628','s', 0.072],
                                [1.391,'#377eb8','v', 0.0427],
                                [1.605,'#984ea3','D',0.0759]],
                          columns = ['freq','colour','marker','rms'])


def parse_txt(txt):
    try: 
        return datetime.strptime(txt, "%Y-%m-%d %H:%M:%S.%f")
    except ValueError:
        return datetime.strptime(txt, "%Y-%m-%d %H:%M:%S")

if os.path.exists(str(sourceID)+'_lightcurve.csv'):
    lightcurve = pd.read_csv(str(sourceID)+'_lightcurve.csv')
    lightcurve['Time'] = lightcurve.apply(lambda row: parse_txt(row.Time), axis=1)
    
else:
    # Connect to the database and run the queries
    session = dbtools.access(engine,host,port,user,password,database)
    #lcurve = []
    lcurve = np.array([]).reshape(0,5)
    for index, row in srcIDs.iterrows():
        print row.freq
        flxVals = session.query(Assocxtrsource,Extractedsource).select_from(join(Assocxtrsource,Extractedsource)).filter(Assocxtrsource.runcat_id == row.srcID).all()
        lightcurve = pd.DataFrame(data=[[flxVals[x].Extractedsource.image.id, flxVals[x].Extractedsource.f_int, flxVals[x].Extractedsource.f_int_err] for x in range(len(flxVals))], columns = ['Image','Flux','FluxErr'])

        images= session.query(Image).all()
        images = pd.DataFrame(data=[[images[x].id,images[x].taustart_ts,np.around(images[x].band.freq_central/1e9, decimals=3)] for x in range(len(images))], columns=['Image','Time','Freq'])

        TMP= pd.merge(lightcurve, images, on="Image")
        lightcurveTMP = TMP.to_numpy()
        lcurve = np.concatenate((lcurve,lightcurveTMP))
    lightcurve = pd.DataFrame(lcurve, columns=['Image','Flux','FluxErr','Time','Freq'])
    lightcurve.to_csv(str(sourceID)+'_lightcurve.csv')


# setting up the plot
nullfmt   = NullFormatter()         # no labels
fontP = FontProperties()
fontP.set_size('xx-large')
fig, axs = plt.subplots(4, sharex=True, sharey=True,figsize=(8,16))

axs[0].set_title(title, fontsize=14)

ymxs = []
for index, row in freqs.iterrows():
    lightcurveTMP = lightcurve.loc[lightcurve['Freq'] == row.freq]
    axs[index].errorbar(lightcurveTMP.Time,lightcurveTMP.Flux*1e3, yerr=lightcurveTMP.FluxErr*1e3, c='b', fmt='o', markersize=7, linestyle=':')
    axs[index].set_ylabel('Flux Density (mJy)', fontsize=14)
    ymin,ymax = axs[index].get_ylim()
    ymxs.append(ymax)

for index, row in freqs.iterrows():    
    axs[index].text( max(lightcurveTMP.Time),max(ymxs)*0.9,str(row.freq)+' GHz',ha='right', fontsize=14)

# Hide x labels and tick labels for all but bottom plot.
for ax in axs:
    ax.label_outer()
    
plt.subplots_adjust(wspace=0, hspace=0.06)
plt.xlabel('Date', fontsize=14)
plt.savefig(outname)
