# plotLightcurve.py
#
# a code to plot a multi-frequency lightcurve for a source

# Import all the dependencies and generic setup
import os
import scipy as sp
import numpy as np
import pandas as pd
from sqlalchemy import *
from sqlalchemy.orm import relationship
import tkp.db
import logging
logging.basicConfig(level=logging.INFO)
query_loglevel = logging.WARNING  # Set to INFO to see queries, otherwise WARNING
import sys
sys.path.append('../')
from dblogin import * # This file contains all the variables required to connect to the database
from databaseTools import dbtools
from tools import tools
from plotting import plot_varib_params as pltvp
import matplotlib
import matplotlib.pyplot as plt
import pylab
pylab.rcParams['legend.loc'] = 'best'
from matplotlib.ticker import NullFormatter
from matplotlib.font_manager import FontProperties
from astropy import units as u
from astropy.coordinates import SkyCoord
from datetime import datetime

# Needed for accessing tkp database directly. Comment out this and lines 54-62 to use without installing tkp.
from tkp.db.model import Extractedsource
from tkp.db.model import Assocxtrsource
from tkp.db.model import Image

# The input database, dataset and thresholds
dataset = 25
sourceID = 59615
title = 'MKT J182001.4+072945'

freqs = pd.DataFrame(data=[[0.963,'#e41a1c','o',0.0838],
                                [1.177,'#a65628','s', 0.072],
                                [1.391,'#377eb8','v', 0.0427],
                                [1.605,'#984ea3','D',0.0759]],
                          columns = ['freq','colour','marker','rms'])
database = 'AR_testing4'
outname = str(sourceID)+'_lightcurve.png' # name of the plotdata datafile

def parse_txt(txt):
    try: 
        return datetime.strptime(txt, "%Y-%m-%d %H:%M:%S.%f")
    except ValueError:
        return datetime.strptime(txt, "%Y-%m-%d %H:%M:%S")

if os.path.exists(str(sourceID)+'_lightcurve.csv'):
    lightcurve = pd.read_csv(str(sourceID)+'_lightcurve.csv')
    lightcurve['Time'] = lightcurve.apply(lambda row: parse_txt(row.Time), axis=1)
    
else:
    # Connect to the database and run the queries
    session = dbtools.access(engine,host,port,user,password,database)
    flxVals = session.query(Assocxtrsource,Extractedsource).select_from(join(Assocxtrsource,Extractedsource)).filter(Assocxtrsource.runcat_id == sourceID).all()
    lightcurve = pd.DataFrame(data=[[flxVals[x].Extractedsource.image.id, flxVals[x].Extractedsource.f_int, flxVals[x].Extractedsource.f_int_err, flxVals[x].Extractedsource.det_sigma] for x in range(len(flxVals))], columns = ['Image','Flux','FluxErr','SNR'])
    images= session.query(Image).all()
    images = pd.DataFrame(data=[[images[x].id,images[x].taustart_ts,np.around(images[x].band.freq_central/1e9, decimals=3)] for x in range(len(images))], columns=['Image','Time','Freq'])
    lightcurve = pd.merge(lightcurve, images, on="Image")
    lightcurve.to_csv(str(sourceID)+'_lightcurve.csv')


# setting up the plot
nullfmt   = NullFormatter()         # no labels
fontP = FontProperties()
fontP.set_size('xx-large')
fig, axs = plt.subplots(4, sharex=True, sharey=True,figsize=(8,16))

axs[0].set_title(title, fontsize=14)
    
for index, row in freqs.iterrows():
    lightcurveTMP = lightcurve.loc[lightcurve['Freq'] == row.freq]
    axs[index].errorbar(lightcurveTMP.Time,lightcurveTMP.Flux*1e3, yerr=lightcurveTMP.FluxErr*1e3, c='b', fmt='o', markersize=7, linestyle=':')
    axs[index].axhline(y=row.rms, color='k', linestyle='--')
    axs[index].set_ylabel('Flux Density (mJy)', fontsize=14)
    ymin,ymax = axs[index].get_ylim()
    axs[index].text( max(lightcurveTMP.Time),ymax*0.85,str(row.freq)+' GHz',ha='right', fontsize=14)

# Hide x labels and tick labels for all but bottom plot.
for ax in axs:
    ax.label_outer()
    
plt.subplots_adjust(wspace=0, hspace=0.06)
plt.xlabel('Date', fontsize=14)
plt.savefig(outname)

exit()
    
    
